use clap::ValueEnum;
use clap::{Parser, Subcommand};
use anyhow::Result;
use easee::api;
use easee::stream;

use tracing::info;

#[derive(Debug,Clone,ValueEnum)]
enum Command {
    Start,
    Stop,
    Pause,
    Resume,
}

#[derive(Debug,Clone,Subcommand)]
enum Mode {
    Status { id: Option<String> },
    Ongoing,
    Latest,
    Stream,
    Command { id: String, command: Command }
}

#[derive(Debug,Parser)]
struct CLI {
    #[arg(short,long,env)]
    username: String,
    #[arg(short,long,env)]
    password: String,
    #[command(subcommand)]
    mode: Mode,
}

fn main() -> Result<()> {
    
    tracing::subscriber::set_global_default(tracing_subscriber::FmtSubscriber::new())
        .expect("Tracing subscriber failed");

    let args = CLI::parse();
    let mut ctx = api::Context::from_login(&args.username, &args.password)?;
    
    info!("Logged in");

    let sites = ctx.sites()?;
    let chargers = ctx.chargers()?;
    info!("{} sites and {} chargers available", sites.len(), chargers.len());

    match args.mode {
        Mode::Status { id } => {
            for c in &chargers {
                if id.as_deref().map(|id| id == &c.id).unwrap_or(true) {
                    println!("{}: {:?}", c.id, c.state(&mut ctx));
                }       
            }
            Ok(())
        },
        Mode::Ongoing => {
            for c in &chargers {
                println!("{}: {:?}", c.id, c.ongoing_session(&mut ctx));
            }
            Ok(())
        },
        Mode::Latest => {
            for c in &chargers {
                println!("{:?}", c.latest_session(&mut ctx));
            }
            Ok(())
        },
        Mode::Stream => {
            let mut stream = stream::Stream::open(&mut ctx)?;
            for c in &chargers {
                stream.subscribe(&c.id)?;
            }

            let mut stream = easee::signalr::Stream::from_ws(stream);
            loop {
                println!("{:?}", stream.recv()?);
            }
        },
        Mode::Command { id, command } => {

            for c in &chargers {
                if c.id == id {
                    match command {
                        Command::Start => c.start(&mut ctx)?,
                        Command::Stop => c.stop(&mut ctx)?,
                        Command::Pause => c.pause(&mut ctx)?,
                        Command::Resume => c.resume(&mut ctx)?,
                    }
                    return Ok(())
                }
            }

            eprintln!("Charger not found.");
            Ok(())
            
        },

        
    }



}
